/* --------------------------------------------------------
 *  IR Transmitter module
 *
 *    Tx buffer ( <-shift out )
 *      |1|    |        | |0|
 *       | +--- +------- | |
 *       | |    |        | stop bit
 *       | |    |        odd parity
 *       | |    data[7:0]
 *       | id code[3:0]
 *       start bit
 *
 *	(tx.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

`define IDLE 1'b0
`define TX   1'b1

module IR_transmitter(clock, reset_N, start_N, id, data, s_out);
    input        clock;		// 1/512 [MHz] = 1.953[KHz]
    input        reset_N;	// reset (active low)
    input        start_N; 	// start (active low)
    input  [3:0] id;		// identify code
    input  [7:0] data;		// transmition data
    output       s_out;		// serial output data
    
    reg [14:0] buffer;		// serial out data 
    reg [ 3:0] count;		// shift out count	
    reg        state;		// state

    assign s_out = buffer[14];
    
    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin
            state  <= `IDLE;
            buffer <= 15'h0000;
        end else if((state==`IDLE) && !start_N) begin
            state  <= `TX;
            count  <= 4'b0000;
            buffer <= {1'b1, id, data, ~^{id, data}, 1'b0};
        end else if(state==`TX) begin
            buffer <= {buffer[13:0], 1'b0};
            count  <= count + 1;
            if(count == 4'b1110) begin
	        state <= `IDLE;	// wait until next start
            end
        end
    end

endmodule

