/* ext_dymc_on.v ----------------------------------------------------
    LED dynamic light on control for SWLED2006 extension board
						(C)2006 kim_syn
  -------------------------------------------------------------------*/

`define HIGH 1'b1
`define LOW  1'b0

module ext_dymc_on_ctrl(ctrl, md, row_N, extsegsel_N, extledon_N);
    input  [ 3:0] ctrl;		// control input
    input  [ 3:0] md;		// operation mode
    output [15:0] row_N;	// anode pin (source side)
    output [ 3:0] extsegsel_N;	// external 7segLED dirve (active low)
    output        extledon_N;	// external LED drive (active low)

    reg    [ 3:0] d_count; 
    wire          clk;		// dynamic light on clock
    wire          rst_N;	// dynamic light on digit register reset
    

// MD[0123]=[3210]  function
//    1111   1111   7seg auto mode
//    1110   0111   7seg selt mode
//    1101   1011   matrix auto mode
//    -0--   --0-   LED & 7seg test
//    0---   ---0   matrix test

  // external LEDs dirve
    assign extledon_N = ((md[1]==1'b0) || (md[2:0]==3'b111)) ? `LOW: `HIGH;  
    
  // external 7seg LEDs dynamic light on control
    assign extsegsel_N = (md[1]==1'b0) ?                           4'b0000:	// test
                         (md==4'b1111) ? ( (d_count[1:0]==2'b00) ? 4'b1110:
                                           (d_count[1:0]==2'b01) ? 4'b1101:
                                           (d_count[1:0]==2'b10) ? 4'b1011:
                                           (d_count[1:0]==2'b11) ? 4'b0111:
                                                                   4'b1111):
                         (md==4'b0111) ?                           ctrl:	// self
                                                                   4'b1111;	// off

  // LED dot matrix dynamic drive control
    assign row_N = (md[0]==1'b0) ?                   16'h0000:	// test
                   (md==4'b1011) ? ( d_count==4'h0 ? 16'hFFFE:
                                     d_count==4'h1 ? 16'hFFFD:
                                     d_count==4'h2 ? 16'hFFFB:
                                     d_count==4'h3 ? 16'hFFF7:
                                     d_count==4'h4 ? 16'hFFEF:
                                     d_count==4'h5 ? 16'hFFDF:
                                     d_count==4'h6 ? 16'hFFBF:
                                     d_count==4'h7 ? 16'hFF7F:
                                     d_count==4'h8 ? 16'hFEFF:
                                     d_count==4'h9 ? 16'hFDFF:
                                     d_count==4'hA ? 16'hFBFF:
                                     d_count==4'hB ? 16'hF7FF:
                                     d_count==4'hC ? 16'hEFFF:
                                     d_count==4'hD ? 16'hDFFF:
                                     d_count==4'hE ? 16'hBFFF:
                                     d_count==4'hF ? 16'h7FFF:
                                                     16'hFFFF):
                                                     16'hFFFF ;

  // counter for dynamic light on control
    assign clk   = ctrl[1];
    assign rst_N = ctrl[0];

    always @(posedge clk or negedge rst_N) begin
        if(!rst_N) begin
            d_count <= 4'b0000;
        end else begin
            d_count <= d_count + 1;
        end
    end

endmodule
