/* --------------------------------------------------------
 *  font ROM for NTSC character display controller
 *	(fontrom.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

module fontROM(reset_N, adrs, data);
    input        reset_N;
    input  [9:0] adrs;
    output [7:0] data;

    reg  [7:0] asciifont [10'b0000000_000:10'b1111111_111];
                  //                  ***--> line number
	  	  //          *******------> character code

    wire [7:0] romdata;

  // output
    assign romdata = adrs[9:3]>=7'b010_0000 ? asciifont[adrs] : 8'h55;

  // reverse bit order of character dot data
    assign data = {romdata[0], romdata[1], romdata[2], romdata[3],
                   romdata[4], romdata[5], romdata[6], romdata[7]};

  // initialize
    always @(negedge reset_N) begin
        if(!reset_N) begin
            asciifont[10'b0110001_000] <= 8'b00000100; // 1 (31)
            asciifont[10'b0110001_001] <= 8'b00001100;
            asciifont[10'b0110001_010] <= 8'b00000100;
            asciifont[10'b0110001_011] <= 8'b00000100;
            asciifont[10'b0110001_100] <= 8'b00000100;
            asciifont[10'b0110001_101] <= 8'b00000100;
            asciifont[10'b0110001_110] <= 8'b00001110;
            asciifont[10'b0110001_111] <= 8'b00000000;

            asciifont[10'b1000001_000] <= 8'b00000100; // A (41)
            asciifont[10'b1000001_001] <= 8'b00001010;
            asciifont[10'b1000001_010] <= 8'b00010001;
            asciifont[10'b1000001_011] <= 8'b00010001;
            asciifont[10'b1000001_100] <= 8'b00011111;
            asciifont[10'b1000001_101] <= 8'b00010001;
            asciifont[10'b1000001_110] <= 8'b00010001;
            asciifont[10'b1000001_111] <= 8'b00000000;

            asciifont[10'b1100001_000] <= 8'b00000000; // a (61)
            asciifont[10'b1100001_001] <= 8'b00001110;
            asciifont[10'b1100001_010] <= 8'b00010001;
            asciifont[10'b1100001_011] <= 8'b00001101;
            asciifont[10'b1100001_100] <= 8'b00010010;
            asciifont[10'b1100001_101] <= 8'b00001111;
            asciifont[10'b1100001_110] <= 8'b00000000;
            asciifont[10'b1100001_111] <= 8'b00000000;

	end else begin
	    // no update operation
	end
    end

endmodule
