/* --------------------------------------------------------
 *  NTSC video synchronous signal generator
 *	(syncgen.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

//  base clock = 10[MHz]

`include "my_const.v"

`define H_PERIOD  634
`define V_PERIOD  261
`define HSYNC_BGN  14
`define HSYNC_END  61
`define VSYNC_BGN   2
`define VSYNC_END   5


`define H_DSP_BGN 143
`define H_DSP_END 599
`define V_DSP_BGN  39
`define V_DSP_END 239


module syncgen(xtal, reset_N, Sync, H_sync, V_sync,
                                    x_count, y_count, x_disp, y_disp);
    input        xtal; 		// 10[MHz] clock
    input        reset_N;	// active low reset
    output       Sync;		// sync signal
    output       H_sync;	// horizontal sync signal
    output       V_sync;	// vertical   sync signal
    output [8:0] x_count;	// horizontal position (0..455)
    output [7:0] y_count;	// virtical   position (0..199)
    output       x_disp;	// horizontal display area
    output       y_disp;	// virtical   display area

    wire [8:0] x_count;
    wire [7:0] y_count;

    reg        Sync, H_sync, V_sync;
    reg        x_disp, y_disp, H_syncd;
    reg  [9:0] clk_count;	// horizontal clock count
    reg  [7:0] h_count;		// virtical   clock count

// horizontal sync and x axis counter
  // basic counter
    always @(posedge xtal or negedge reset_N) begin
        if(!reset_N) begin
            clk_count <= 0;
        end else if(clk_count==`H_PERIOD) begin
            clk_count <= 0;
        end else begin
            clk_count <= clk_count + 1;
        end
    end

  // X posotion count ... x_count = clk_count - `H_DSP_BGN(143) + 1
    assign x_count = {(clk_count[8:4] - 5'b01001), clk_count[3:0]};

  // Hsync generator
    always @(posedge xtal or negedge reset_N) begin
        if(!reset_N) begin
            H_sync <= `OFF;
        end else if(clk_count==`HSYNC_BGN) begin
            H_sync <= `ON;
        end else if(clk_count==`HSYNC_END) begin
            H_sync <= `OFF;
        end
    end

  // horizontal display area signal
    always @(posedge xtal or negedge reset_N) begin
        if(!reset_N) begin
            x_disp  <= `OFF;
        end else if(clk_count==`H_DSP_BGN) begin
            x_disp  <= `ON;
        end else if(clk_count==`H_DSP_END) begin
            x_disp  <= `OFF;
        end else begin
        end
    end

// virtical sync and y axis counter
  // basic counter
    always @(posedge xtal or negedge reset_N) begin
        if(!reset_N) begin
            h_count <= 0;
        end else if(H_sync & ~H_syncd) begin
            if(h_count==`V_PERIOD) begin
                h_count <= 0;
            end else begin
                h_count <= h_count + 1;
            end
        end
    end

  // Y posotion count ... y_count = h_count - `V_DSP_BGN(39) + 1
    assign y_count = {(h_count[7:3] - 5'b00101), h_count[2:0]};

  // Vsync generator
    always @(posedge xtal or negedge reset_N) begin
        if(!reset_N) begin
            V_sync <= `OFF;
        end else if(h_count==`VSYNC_BGN) begin
            V_sync <= `ON;
        end else if(h_count==`VSYNC_END) begin
            V_sync <= `OFF;
        end
    end

  // vertical display area signal
    always @(posedge xtal) begin
        H_syncd <= H_sync;
    end

    always @(posedge xtal or negedge reset_N) begin
        if(!reset_N) begin
            y_disp  <= `OFF;
        end else if(H_sync & ~H_syncd) begin
	    if(h_count==`V_DSP_BGN) begin
                y_disp  <= `ON;
            end else if(h_count==`V_DSP_END) begin
                y_disp  <= `OFF;
            end else begin
            end
        end
    end

// sync mixer
    always @(posedge xtal) begin
        Sync <= H_sync ~^ V_sync;
    end

endmodule
