/* --------------------------------------------------------
 *  barrel rotater (multiplexer version)
 *	(barrel_mux.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

module barrel_rotate16(l_r, sft, in, rslt);
    input         l_r;  // 1=left, 0=right
    input  [ 3:0] sft;  // shift no.
    input  [15:0] in;   // input data
    output [15:0] rslt; // rotated data

    wire   [15:0] s1;	// 0/1 rotated inter midiate signal
    wire   [15:0] s2;	// 0/2 rotated inter midiate signal
    wire   [15:0] s4;	// 0/4 rotated inter midiate signal

    genvar i;	// loop variable for generate-for

  // 0-1 shift
    generate
        for(i=0; i<16; i=i+1) begin : loop1
            assign s1[i]   = sft[0] ? (l_r ? in[(i-1)&4'hF]: in[(i+1)&4'hF]): in[i];
        end
    endgenerate

  // 0-2 shift
    generate
        for(i=0; i<16; i=i+1) begin : loop2
            assign s2[i]   = sft[1] ? (l_r ? s1[(i-2)&4'hF]: s1[(i+2)&4'hF]): s1[i];
        end
    endgenerate

  // 0-4 shift
    generate
        for(i=0; i<16; i=i+1) begin : loop3
            assign s4[i]   = sft[2] ? (l_r ? s2[(i-4)&4'hF]: s2[(i+4)&4'hF]): s2[i];
        end
    endgenerate

  // 0-8 shift
    generate
        for(i=0; i<16; i=i+1) begin : loop4
            assign rslt[i] = sft[3] ? (l_r ? s4[(i-8)&4'hF]: s4[(i+8)&4'hF]): s4[i];
        end
    endgenerate

endmodule
