/* --------------------------------------------------------
 *  barrel rotater (multiplexer version)
 *	(barrel_mux.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

module barrel_rotate8(l_r, sft, in, rslt);
    input        l_r;   // 1=left, 0=right
    input  [2:0] sft;   // shift no.
    input  [7:0] in;    // input data
    output [7:0] rslt;  // rotated data

    wire   [7:0] s1;	// 0/1 rotated inter midiate signal
    wire   [7:0] s2;	// 0/2 rotated inter midiate signal

  // 0-1 rotate
    assign s1[0]   = sft[0] ? (l_r ? in[7]: in[1]): in[0];
    assign s1[1]   = sft[0] ? (l_r ? in[0]: in[2]): in[1];
    assign s1[2]   = sft[0] ? (l_r ? in[1]: in[3]): in[2];
    assign s1[3]   = sft[0] ? (l_r ? in[2]: in[4]): in[3];
    assign s1[4]   = sft[0] ? (l_r ? in[3]: in[5]): in[4];
    assign s1[5]   = sft[0] ? (l_r ? in[4]: in[6]): in[5];
    assign s1[6]   = sft[0] ? (l_r ? in[5]: in[7]): in[6];
    assign s1[7]   = sft[0] ? (l_r ? in[6]: in[0]): in[7];

  // 0-2 rotate
    assign s2[0]   = sft[1] ? (l_r ? s1[6]: s1[2]): s1[0];
    assign s2[1]   = sft[1] ? (l_r ? s1[7]: s1[3]): s1[1];
    assign s2[2]   = sft[1] ? (l_r ? s1[0]: s1[4]): s1[2];
    assign s2[3]   = sft[1] ? (l_r ? s1[1]: s1[5]): s1[3];
    assign s2[4]   = sft[1] ? (l_r ? s1[2]: s1[6]): s1[4];
    assign s2[5]   = sft[1] ? (l_r ? s1[3]: s1[7]): s1[5];
    assign s2[6]   = sft[1] ? (l_r ? s1[4]: s1[0]): s1[6];
    assign s2[7]   = sft[1] ? (l_r ? s1[5]: s1[1]): s1[7];

  // 0-4 rotate
    assign rslt[0] = sft[2] ? (l_r ? s2[4]: s2[4]): s2[0];
    assign rslt[1] = sft[2] ? (l_r ? s2[5]: s2[5]): s2[1];
    assign rslt[2] = sft[2] ? (l_r ? s2[6]: s2[6]): s2[2];
    assign rslt[3] = sft[2] ? (l_r ? s2[7]: s2[7]): s2[3];
    assign rslt[4] = sft[2] ? (l_r ? s2[0]: s2[0]): s2[4];
    assign rslt[5] = sft[2] ? (l_r ? s2[1]: s2[1]): s2[5];
    assign rslt[6] = sft[2] ? (l_r ? s2[2]: s2[2]): s2[6];
    assign rslt[7] = sft[2] ? (l_r ? s2[3]: s2[3]): s2[7];

endmodule
