/* --  V~[V  ------------------------------
 *  static RAM
 *	(sramsim.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

module sramtest();
    reg  [ 9:0] adrs;
    reg  [11:0] wdata;
    reg         cs, rd, wr;
    wire [11:0] data;

  // sram module instantiation
    sram #(4, 8, 256) sram1 (adrs[7:0], data[ 3:0], cs, rd, wr);
    sram #(8,10,1024) sram2 (adrs,      data[11:4], cs, rd, wr);

  // data bus 
    assign data = wr ? wdata : 12'hZZZ;

  // sram initilaize
    initial begin
        $readmemb("data1.bin", sram1.mem);
        $readmemb("data2.bin", sram2.mem);
    end

  // monitoring signal
    initial begin
        $monitor("%4t: %h, %h, %h, %b, %b, %b",
                  $time, adrs, data[11:4], data[3:0], cs, rd, wr);
    end

  // test
    initial begin
      // read test
        adrs <= 10'h000; wdata <= 12'h000; cs <= 0; rd <= 0; wr <= 0;
      #10
        adrs <= 10'h000; wdata <= 12'h000; cs <= 1; rd <= 1; wr <= 0;
      #10
        adrs <= 10'h001; wdata <= 12'h000; cs <= 1; rd <= 1; wr <= 0;
      #10
        adrs <= 10'h002; wdata <= 12'h000; cs <= 1; rd <= 0; wr <= 0;
      #10
        adrs <= 10'h002; wdata <= 12'h000; cs <= 1; rd <= 1; wr <= 0;
      #10
        adrs <= 10'h002; wdata <= 12'h000; cs <= 0; rd <= 1; wr <= 0;
      #10

      // write test
        adrs <= 10'h101; wdata <= 12'h123; cs <= 1; rd <= 0; wr <= 0;
      #10
        adrs <= 10'h101; wdata <= 12'h123; cs <= 1; rd <= 0; wr <= 1;
      #10
        adrs <= 10'h101; wdata <= 12'h123; cs <= 1; rd <= 0; wr <= 0;
      #10
        adrs <= 10'h101; wdata <= 12'h000; cs <= 1; rd <= 1; wr <= 0;
      #10
        adrs <= 10'h101; wdata <= 12'h345; cs <= 0; rd <= 0; wr <= 0;
      #10
        adrs <= 10'h101; wdata <= 12'h345; cs <= 0; rd <= 0; wr <= 1;
      #10
        adrs <= 10'h101; wdata <= 12'h345; cs <= 0; rd <= 0; wr <= 0;
      #10
        adrs <= 10'h101; wdata <= 12'h345; cs <= 1; rd <= 1; wr <= 0;
      #10
        adrs <= 10'h122; wdata <= 12'h345; cs <= 0; rd <= 0; wr <= 0;
      #10
        adrs <= 10'h122; wdata <= 12'h345; cs <= 1; rd <= 0; wr <= 1;
      #10
        adrs <= 10'h122; wdata <= 12'h567; cs <= 1; rd <= 0; wr <= 1;
      #10
        adrs <= 10'h133; wdata <= 12'h567; cs <= 1; rd <= 0; wr <= 1;
      #10
        adrs <= 10'h133; wdata <= 12'h000; cs <= 0; rd <= 0; wr <= 0;
      #10
        adrs <= 10'h122; wdata <= 12'h000; cs <= 1; rd <= 0; wr <= 0;
      #10
        adrs <= 10'h122; wdata <= 12'h000; cs <= 1; rd <= 1; wr <= 0;
      #10
        adrs <= 10'h133; wdata <= 12'h000; cs <= 1; rd <= 1; wr <= 0;
      #10
        adrs <= 10'h133; wdata <= 12'h000; cs <= 1; rd <= 0; wr <= 0;
      #10
        $finish;
    end
endmodule
