/* --------------------------------------------------------
 *  multiplier (non stop control version)
 *	(multi4nc.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

`define ON  1
`define OFF 0

module multiplier(clock, reset_N, wrstb_N, MLsel, sw_data, start_N, Hreg, Lreg);

  //  {Hreg, Lreg} <= Lreg * Mreg

    input              clock, reset_N, wrstb_N, MLsel, start_N;
    input  [3:0] sw_data;
    output [4:0] Hreg;
    output [3:0] Lreg;
    
    reg    [4:0] Hreg;
    reg    [3:0] Lreg, Mreg;
    reg          Istate, Astate, Sstate;
    
    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin		// reset
            Hreg   <= 0;
            Lreg   <= 0;
            Mreg   <= 0;
            Istate <= `ON;
            Astate <= `OFF;
            Sstate <= `OFF;
        end else if(Istate) begin
            if(!wrstb_N) begin
                case(MLsel)		// initial data setting
                  0: Lreg <= sw_data;
                  1: Mreg <= sw_data;
                endcase
            end else if(!start_N) begin
                Hreg   <= 0;
                Istate <= `OFF;
                Astate <= `ON;
            end else begin
                /* no change */
            end
        end else if(Astate) begin	// add state
            Hreg   <= Hreg + (Lreg[0] ? Mreg : 4'b0000);
            Astate <= `OFF;
            Sstate <= `ON;
        end else if(Sstate) begin	// shift state
            {Hreg, Lreg} <= {1'b0, Hreg, Lreg[3:1]};
            Sstate <= `OFF;
            Astate <= `ON;
        end else begin			// illegal state
            Istate <= `OFF;
            Astate <= `OFF;
            Sstate <= `OFF;
        end
    end
endmodule
