/* --------------------------------------------------------
 *  multiplier (stop control version)
 *	(multi4sc.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

`define ON  1
`define OFF 0

module multiplier(clock, reset_N, wrstb_N, MLsel, sw_data,
                  start_N, ready_N, Hreg, Lreg);

  //  {Hreg, Lreg} <= Lreg * Mreg

    input        clock, reset_N, wrstb_N, MLsel, start_N;
    input  [3:0] sw_data;
    output       ready_N;
    output [4:0] Hreg;
    output [3:0] Lreg;
    
    reg    [4:0] Hreg;
    reg    [3:0] Lreg, Mreg;
    reg          Istate, Astate, Sstate;
    reg          ready_N;
    reg    [1:0] counter;	// loop counter
    
    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin		// reset
            Hreg    <= 0;
            Lreg    <= 0;
            Mreg    <= 0;
            ready_N <= 0;		// status = ready
            counter <= 0;
            Istate  <= `ON;
            Astate  <= `OFF;
            Sstate  <= `OFF;
        end else if(Istate) begin
            if(!wrstb_N) begin
                case(MLsel)		// initial data setting
                  0: Lreg <= sw_data;
                  1: Mreg <= sw_data;
                endcase
            end else if(!start_N) begin
                ready_N <= 1;		// status = not ready
                Hreg    <= 0;
                Istate  <= `OFF;
                Astate  <= `ON;
            end else begin
                /* no change */
            end
        end else if(Astate) begin
            Hreg   <= Hreg + (Lreg[0] ? Mreg : 4'b0000);
            Astate <= `OFF;
            Sstate <= `ON;
        end else if(Sstate) begin
            {Hreg, Lreg} <= {1'b0, Hreg, Lreg[3:1]};
            Sstate <= `OFF;
            counter <= counter + 1;
            if(counter==3) begin	// operation end
                ready_N <= 0;		// status = ready
                Istate  <= `ON;
            end else begin		// operation continue
                Astate  <= `ON;
            end
        end else begin // illegal state
            Istate <= `OFF;
            Astate <= `OFF;
            Sstate <= `OFF;
        end
    end
endmodule
