/* --------------------------------------------------------
 *  register alu (register transfer description)
 *	(exfsm.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

`define ADD 0
`define SUB 1
`define AND 2
`define OR  3

module ralu(clock, data, we_N, xwr_N, xrd_N, 
                   regsel_l, regsel_r, regsel_w, aluop);
    input       clock, we_N;
    inout [7:0] data;
    input       xwr_N, xrd_N;
    input [2:0] regsel_l, regsel_r, regsel_w;
    input [1:0] aluop;

    reg   [7:0] GR [7:0];

    assign data = xrd_N ? 8'hZZ  : GR[regsel_l];

    always @(posedge clock) begin
        if(!we_N && xwr_N) begin
            case (aluop)
              `ADD : GR[regsel_w] <= GR[regsel_l] + GR[regsel_r];
              `SUB : GR[regsel_w] <= GR[regsel_l] - GR[regsel_r];
              `AND : GR[regsel_w] <= GR[regsel_l] & GR[regsel_r];
              `OR  : GR[regsel_w] <= GR[regsel_l] | GR[regsel_r];
            endcase
        end else if(!we_N && !xwr_N)begin
            GR[regsel_w] <= data;
        end
    end

endmodule
