/* --------------------------------------------------------
 *  register alu (net list description)
 *	(netlist.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

`define ADD 0
`define SUB 1
`define AND 2
`define OR  3

module ralu(clock, data, we_N, xwr_N, xrd_N, 
                   regsel_l, regsel_r, regsel_w, aluop);
    input       clock, we_N;
    inout [7:0] data;
    input       xwr_N, xrd_N;
    input [2:0] regsel_l, regsel_r, regsel_w;
    input [1:0] aluop;

    wire  [7:0] lbus, rbus, wbus, aluout;

    reg   [7:0] GR [7:0];

    assign lbus = (regsel_l==0) ? GR[0] : 8'hZZ;
    assign lbus = (regsel_l==1) ? GR[1] : 8'hZZ;
    assign lbus = (regsel_l==2) ? GR[2] : 8'hZZ;
    assign lbus = (regsel_l==3) ? GR[3] : 8'hZZ;
    assign lbus = (regsel_l==4) ? GR[4] : 8'hZZ;
    assign lbus = (regsel_l==5) ? GR[5] : 8'hZZ;
    assign lbus = (regsel_l==6) ? GR[6] : 8'hZZ;
    assign lbus = (regsel_l==7) ? GR[7] : 8'hZZ;

    assign rbus = (regsel_r==0) ? GR[0] : 8'hZZ;
    assign rbus = (regsel_r==1) ? GR[1] : 8'hZZ;
    assign rbus = (regsel_r==2) ? GR[2] : 8'hZZ;
    assign rbus = (regsel_r==3) ? GR[3] : 8'hZZ;
    assign rbus = (regsel_r==4) ? GR[4] : 8'hZZ;
    assign rbus = (regsel_r==5) ? GR[5] : 8'hZZ;
    assign rbus = (regsel_r==6) ? GR[6] : 8'hZZ;
    assign rbus = (regsel_r==7) ? GR[7] : 8'hZZ;

    assign data = xrd_N ? 8'hZZ  : lbus;

//    assign wbus = xwr_N ? aluout : 8'hZZ;
//    assign wbus = xwr_N ? 8'hZZ  : data;

    assign wbus = xwr_N ? aluout : data;

    assign aluout = alu(lbus, rbus, aluop);
    
    function [7:0] alu;
        input [7:0] lbus, rbus;
        input [1:0] aluop;
        
        begin
            case (aluop)
              `ADD: alu = lbus + rbus;
              `SUB: alu = lbus - rbus;
              `AND: alu = lbus & rbus;
              `OR : alu = lbus | rbus;
            endcase
        end
    endfunction

    always @(posedge clock) begin
        if(!we_N) begin
            GR[regsel_w] <= wbus;
        end
    end

endmodule
