/* --------------------------------------------------------
 *  signal controller (simplicity version)
 *	(signal_smpl.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

`define RR0 3'b000
`define RG  3'b001
`define RY  3'b010
`define RR1 3'b100
`define GR  3'b101
`define YR  3'b110

`define RR_time 1	// = 1+1
`define G_time  5	// = 5+1
`define Y_time  2	// = 2+1

module signal(clock, reset_N, gyr1, gyr0);
    input        clock;
    input        reset_N;
    output [2:0] gyr1, gyr0;

    reg [2:0] state;
    reg [5:0] t_count;

    assign gyr0 = (state==`RG) ? 3'b100 :
                  (state==`RY) ? 3'b010 : 3'b001;

    assign gyr1 = (state==`GR) ? 3'b100 :
                  (state==`YR) ? 3'b010 : 3'b001;

    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin
            state   <= `RR0;
            t_count <= `RR_time;
        end else if(state==`RR0) begin
            if(t_count==0) begin
                state   <= `RG;
                t_count <= `G_time;
            end else begin
                t_count <= t_count - 1;
            end
        end else if(state==`RG) begin
            if(t_count==0) begin
                state   <= `RY;
                t_count <= `Y_time;
            end else begin
                t_count <= t_count - 1;
            end
        end else if(state==`RY) begin
            if(t_count==0) begin
                state   <= `RR1;
                t_count <= `RR_time;
            end else begin
                t_count <= t_count - 1;
            end
        end else if(state==`RR1) begin
            if(t_count==0) begin
                state   <= `GR;
                t_count <= `G_time;
            end else begin
                t_count <= t_count - 1;
            end
        end else if(state==`GR) begin
            if(t_count==0) begin
                state   <= `YR;
                t_count <= `Y_time;
            end else begin
                t_count <= t_count - 1;
            end
        end else if(state==`YR) begin
            if(t_count==0) begin
                state   <= `RR0;
                t_count <= `RR_time;
            end else begin
                t_count <= t_count - 1;
            end
        end
    end

endmodule
