/* -- Xilinx XC95108 _pgbvEW[ ---------
 *  top module for slot machine
 *      XC95top2001.v		designed by Shinya KIMURA
 * -------------------------------------------------------- */
`default_nettype none

//`define PS2
//`define LED_MATRIX

module XC95top(
    input         XTAL,		// X'tal OSC clock / NE555 clock
    input         MANCLK,	// manual clock    / NE555 clock
    input         RESET_N,	// reset (active low)
    input  [ 7:0] DIPSW,	// main board DIP switch
    input  [ 7:0] TGLSW,	// toggle switch with chattering
                                // tglsw[3:0] = on main board,
                                // tglsw[7:4] = on ext. board
    input  [ 3:0] PSHSW,	// push switch with chattering
    input  [ 3:0] TGLCFSW,	// toggle switch chattering free
    output [ 7:0] LED_N,	// main board LED
    output [ 7:0] SEG_N,	// main board 7-segment LED
                                //   [ 7:0] = {dot,g,f,e,d,c,b,a}
    output [ 1:0] SEGSEL_N,	// main board 7-segment LED select
                                //   [ 1:0] = {left, right}
    output [15:0] EXTLED_N,	// extension board LED
                                //   [15:0] = LED matrix column
                                //          = {right,  ... left}
                                //   [15:8] = 7-segment LED
                                //          = {dot,g,f,e,d,c,b,a}
                                //   [ 7:0] = single LED
    output [ 3:0] EXTSEGSEL_N,	// extension  7-segment LED select
                                //   [ 3:0] = {left, ... right} for 7-seg. LED
                                //   [   1] = matrix clock for matrix LED
                                //   [   0] = matrix reset for matrix LED
`ifdef PS2 // for PS/2 interface
    input         PS2CLKIN,	// PS/2 mouse/keyboard clock input
    input         PS2DATAIN,	// PS/2 mouse/keyboard data  input
    output        PS2CLKOUT,	// PS/2 mouse/keyboard clock output
    output        PS2DATAOUT	// PS/2 mouse/keyboard data  output
`else      // for stepping motor control interface
    output        STPMT_A,	// step motor dirve (phase A+)
    output        STPMT_A_N,	//                  (phase A-)
    output        STPMT_B,	//                  (phase B+)
    output        STPMT_B_N	//                  (phase B-)
`endif
				);

  // internal signal (active high)
    wire [ 7:0] led;
    wire [ 7:0] seg;
    wire [ 1:0] segsel;
    wire [15:0] extled;
    wire [ 3:0] extsegsel;

  // additional internal signals
    wire clk1KHz, clk10Hz;

  // active level change for positive logic
    assign LED_N       = ~led;
    assign SEG_N       = ~seg;
    assign SEGSEL_N    = ~segsel;
    assign EXTLED_N    = ~extled;
    assign EXTSEGSEL_N = ~extsegsel;

`ifdef LED_MATRIX
    wire [15:0] mtrx_clm;		// matrix column deta [0:15] !!!
    wire        mtrx_clk;		// row counter clock
    wire        mtrx_rst_N;		// row counter reset

    assign extled       = mtrx_clm;
    assign extsegsel[1] = mtrx_clk;
    assign extsegsel[0] = mtrx_rst_N;
`else
    wire [ 7:0] extseg;

    assign extled[15:8] = extseg;	// = {dot, g, f, e, d, c, b, a}
`endif

  // core module instantiation and additional logic
    clock_gen S0(XTAL,		// 1[MHz] base clock
                 clk1KHz,	// 1[KHz] clock for 7-seg LED dynamic light on
                 clk10Hz);	// 10[Hz] clock for slot machine game

    system    S1(clk10Hz,	// game clock
                 clk1KHz,	// dynamic light on clock
                 RESET_N,	// reset
                 ~PSHSW[3],	// game start switch
                 ~PSHSW[2:0],	// stop switch for each digit
                 extsegsel,	// external segment LED select
                 extseg,	// external segment LED
                 extled[7:0]);	// fever LED

endmodule
