/* --------------------------------------------------------
 *  dynamic light on control for slot machine game system
 *	(dynamic.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

module dynamic(d_clock, left, middle, right, digit, d_dgt);
    input        d_clock;
    input  [2:0] left, middle, right;
    output [3:0] digit;
    output [2:0] d_dgt;

    reg  [1:0] d_count;

  // 00->01->10->00 ... counter for dynamic light on control
    always @(posedge d_clock) begin
        if(d_count==2'b10) begin
            d_count <= 2'b00;
        end else begin
            d_count <= d_count + 1;
        end
    end

  // digit select signal
    assign digit = (d_count==2'b00) ? 4'b0001 :
                   (d_count==2'b01) ? 4'b0010 :
                   (d_count==2'b10) ? 4'b0100 : 4'b0000;

  // multiplexer 
    assign d_dgt = (d_count==2'b00) ? right  :
                   (d_count==2'b01) ? middle :
                   (d_count==2'b10) ? left   : 3'b111;

endmodule
