/* --------------------------------------------------------
 *  slot machine game body
 *	(slot_body.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

module slot_body(clock, reset_N, start, stop, left, middle, right, run_stop, fever);
    input        clock, reset_N;
    input        start;
    input  [2:0] stop;
    output [2:0] left, middle, right;
    output       run_stop, fever;

    wire         runL, runM, runR;
    wire         time_out1, time_out2;
    wire         start_trg;
    reg          start_d, start_dd;

    always @(negedge clock) begin
        start_d  <= start;
        start_dd <= start_d;
    end

    assign start_trg = start_d & ~start_dd;

    count8    L (clock, reset_N, runL, left  );
    count8    M (clock, reset_N, runM, middle);
    count8    R (clock, reset_N, runR, right );

    swr_state LS(clock, reset_N, start_trg, (~run_stop & start_trg), stop[2], runL);
    swr_state MS(clock, reset_N, start_trg, time_out1,               stop[1], runM);
    swr_state RS(clock, reset_N, start_trg, time_out2,               stop[0], runR);

    timer     T1(clock, reset_N, (~run_stop & start_trg), time_out1);
    timer     T2(clock, reset_N, time_out1,               time_out2);

  // fever check
    assign run_stop = runL | runM | runR;
    assign fever    = (left==middle) && (left==right);

endmodule
