/* --------------------------------------------------------
 *  stop/wait/run state control for slot machine game
 *	(state.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */
`define STOP 2'b00
`define WAIT 2'b01
`define RUN  2'b10

module swr_state(clock, reset_N, start, run, stop, running);
    input  clock, reset_N;
    input  start, run, stop;
    output running;

    wire  clock, reset_N;
    wire  start, run, stop;
    wire  running;
    reg [1:0] state;

    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin
            state <= `STOP;
        end else if((state==`STOP) && start) begin
            if(~run) begin 
                state <= `WAIT;
            end else begin
                state <= `RUN;
            end
        end else if((state==`WAIT) && run)   begin
            state <= `RUN;
        end else if((state==`RUN)  && stop)  begin
            state <= `STOP;
        end
    end

    assign running = (state==`RUN);
endmodule
