/* --------------------------------------------------------
 *  stepping motor controller ( 2 always @() version)
 *	(stepmtcnt1.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

// phase pulse pattern definition
`define P0state 4'b0001
`define P1state 4'b0100
`define P2state 4'b0010
`define P3state 4'b1000

module stepmtsw(clock, reset_N, start, right, speed, state);
    input        clock, reset_N, start, right;
    input  [1:0] speed;
    output [3:0] state;

    reg    [3:0] state;
    reg    [1:0] wait_state;

  // state sequencer
    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin
            state <= `P0state;
        end else begin
            if(start && (wait_state==2'b00)) begin
                if(right) begin
                    case(state)
                      `P0state: state <= `P1state; 
                      `P1state: state <= `P2state; 
                      `P2state: state <= `P3state; 
                      `P3state: state <= `P0state; 
                    endcase
                end else begin
                    case(state)
                      `P0state: state <= `P3state; 
                      `P1state: state <= `P0state; 
                      `P2state: state <= `P1state; 
                      `P3state: state <= `P2state; 
                    endcase
                end
            end else begin
                state <= state;
            end
        end
    end

  // rotation speed control
    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin
            wait_state <= 2'b00;
        end else if((wait_state==2'b00) && start) begin
            wait_state <= speed;
        end else if( wait_state!=2'b00) begin
            wait_state <= wait_state - 1;
        end
    end

endmodule
