/* --------------------------------------------------------
 *  stepping motor controller (separate module version)
 *	(stepmtcnt2.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

// phase pulse pattern definition
`define P0state 4'b0001
`define P1state 4'b0100
`define P2state 4'b0010
`define P3state 4'b1000

// stepping motor module integration
//
module stepmtsw(clock, reset_N, start, right, speed, state);
    input        clock, reset_N, start, right;
    input  [1:0] speed;
    output [3:0] state;

    wire         go_ahead;
    
    stepmtseq stepmtseq(clock, reset_N, start, right, go_ahead, state);
    speedcont speedcont(clock, reset_N, start, speed, go_ahead);

endmodule


// state sequencer module
//
module stepmtseq(clock, reset_N, start, right, go_ahead, state);
    input        clock, reset_N, start, right, go_ahead;
    output [3:0] state;

    reg    [3:0] state;

    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin
            state <= `P0state;
        end else begin
            if(start && go_ahead) begin
                if(right) begin
                    case(state)
                      `P0state: state <= `P1state; 
                      `P1state: state <= `P2state; 
                      `P2state: state <= `P3state; 
                      `P3state: state <= `P0state; 
                    endcase
                end else begin
                    case(state)
                      `P0state: state <= `P3state; 
                      `P1state: state <= `P0state; 
                      `P2state: state <= `P1state; 
                      `P3state: state <= `P2state; 
                    endcase
                end
            end else begin
                state <= state;
            end
        end
    end
endmodule


// rotation speed control module
//
module speedcont(clock, reset_N, start, speed, go_ahead);
    input        clock, reset_N, start;
    input  [1:0] speed;
    output       go_ahead;

    reg    [1:0] wait_state;

    assign go_ahead = (wait_state==2'b00)? 1 : 0;

    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin
            wait_state <= 2'b00;
        end else if((wait_state==2'b00) && start) begin
            wait_state <= speed;
        end else if(wait_state!=2'b00) begin
            wait_state <= wait_state - 1;
        end
    end
endmodule
