/* --------------------------------------------------------
 *  serial input variable length code decoder  
 *	one-hot (Mealy type)
 *	(vldec_ol1.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

// state set/reset constant
`define ON  1'b1
`define OFF 1'b0

// character code definition
`define A   7'b1000000
`define B   7'b0100000
`define C   7'b0010000
`define D   7'b0001000
`define E   7'b0000100
`define F   7'b0000010
`define G   7'b0000001
`define NON 7'b0000000

module vldec(clock, reset_N, sdata, charsig);
    input  clock, reset_N, sdata;
    output [6:0] charsig;

    reg        S0, S1, S2, S3, S4, S5, SX;
    wire [6:0] charsig;

    assign charsig = chardet(S0, S1, S2, S3, S4, S5, sdata);
    
  // character detection logic
    function [6:0] chardet;
        input  S0, S1, S2, S3, S4, S5, sdata;
        begin
            if(S1) begin
                chardet = (sdata) ? `B   : `A;
            end else if(S3) begin
                chardet = (sdata) ? `D   : `C;
            end else if(S4) begin
                chardet = (sdata) ? `NON : `E;
            end else if(S5) begin
                chardet = (sdata) ? `G   : `F;
            end else begin
                chardet = `NON;
            end
        end
     endfunction

  // state transition control
    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin
            {S0, S1, S2, S3, S4, S5, SX} <= 7'b1000000;
        end else begin
            if(S0) begin
                if(sdata) S2 <= `ON; else S1 <= `ON;
                S0 <= `OFF;
            end else if(S1) begin
                S0 <= `ON;
                S1 <= `OFF;
            end else if(S2) begin
                if(sdata) S4 <= `ON; else S3 <= `ON;
                S2 <= `OFF;
            end else if(S3) begin
                S0 <= `ON;
                S3 <= `OFF;
            end else if(S4) begin
                if(sdata) S5 <= `ON; else S0 <= `ON;
                S4 <= `OFF;
            end else if(S5) begin
                S0 <= `ON;
                S5 <= `OFF;
            end else begin
                SX <= `ON;
            end
        end
    end

endmodule
