/* --------------------------------------------------------
 *  serial input variable length code decoder
 *      one-hot version (Moore type)
 *	(vldec_or2.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

// state set/reset constant
`define ON  1'b1
`define OFF 1'b0

module vldec(clock, reset_N, sdata, charsig);
    input  clock, reset_N, sdata;
    output [6:0] charsig;

    reg        S0, S1, S2, S3, S4, S5, SX;
    reg        Sa, Sb, Sc, Sd, Se, Sf, Sg;
    wire [6:0] charsig;

    assign charsig = {Sa, Sb, Sc, Sd, Se, Sf, Sg};

  // state transition control
    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin
            {S0, S1, S2, S3, S4, S5, SX} <= 7'b1000000;
            {Sa, Sb, Sc, Sd, Se, Sf, Sg} <= 7'b0000000;
        end else begin
            if(S0) begin
                if(sdata) S2 <= `ON; else S1 <= `ON;
                S0 <= `OFF;
            end else if(S1) begin
                if(sdata) Sb <= `ON; else Sa <= `ON;
                S1 <= `OFF;
            end else if(S2) begin
                if(sdata) S4 <= `ON; else S3 <= `ON;
                S2 <= `OFF;
            end else if(S3) begin
                if(sdata) Sd <= `ON; else Sc <= `ON;
                S3 <= `OFF;
            end else if(S4) begin
                if(sdata) S5 <= `ON; else Se <= `ON;
                S4 <= `OFF;
            end else if(S5) begin
                if(sdata) Sg <= `ON; else Sf <= `ON;
                S5 <= `OFF;
            end else if(Sa || Sb || Sc || Sd || Se || Sf || Sg) begin
                if(sdata) S2 <= `ON; else S1 <= `ON;
                {Sa, Sb, Sc, Sd, Se, Sf, Sg} <= 7'b0000000;
            end else begin
                SX <= `ON;
            end
        end
    end

endmodule
