/* --------------------------------------------------------
 *  serial input variable length code decoder
 *      one-hot with non-encoded register version (Mealy type)
 *	(vldec_ol3.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

// state name definition
`define S0  6'b000001
`define S1  6'b000010
`define S2  6'b000100
`define S3  6'b001000
`define S4  6'b010000
`define S5  6'b100000
`define SX  6'b000000

// character code definition
`define A   7'b1000000
`define B   7'b0100000
`define C   7'b0010000
`define D   7'b0001000
`define E   7'b0000100
`define F   7'b0000010
`define G   7'b0000001
`define NON 7'b0000000


module vldec(clock, reset_N, sdata, charsig);
    input  clock, reset_N, sdata;
    output [6:0] charsig;

    reg  [5:0] state;
    wire [6:0] charsig;

    assign charsig = chardet(state, sdata);
    
  // character detection logic
    function [6:0] chardet;
        input  [5:0] state;
        input        sdata;
        begin
            case (state) 
              `S1    : chardet = (sdata) ? `B   : `A;
              `S3    : chardet = (sdata) ? `D   : `C;
              `S4    : chardet = (sdata) ? `NON : `E;
              `S5    : chardet = (sdata) ? `G   : `F;
              default: chardet = `NON;
            endcase
        end
     endfunction

  // state transition control
    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin
            state <= `S0;
        end else begin
            case (state)
              `S0    : state <= (sdata) ? `S2 : `S1;
              `S1    : state <= `S0;
              `S2    : state <= (sdata) ? `S4 : `S3;
              `S3    : state <= `S0;
              `S4    : state <= (sdata) ? `S5 : `S0;
              `S5    : state <= `S0;
              default: state <= `SX;
            endcase
        end
    end
endmodule
