/* --------------------------------------------------------
 *  serial input variable length code decoder
 *      one-hot with non-encoded register version (Moore type)
 *	(vldec_or4.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

// state name definition

`define S0  13'b0000000000001
`define S1  13'b0000000000010
`define S2  13'b0000000000100
`define S3  13'b0000000001000
`define S4  13'b0000000010000
`define S5  13'b0000000100000
`define Sa  13'b1000000000000
`define Sb  13'b0100000000000
`define Sc  13'b0010000000000
`define Sd  13'b0001000000000
`define Se  13'b0000100000000
`define Sf  13'b0000010000000
`define Sg  13'b0000001000000
`define SX  13'b0000000000000


module vldec(clock, reset_N, sdata, charsig);
    input  clock, reset_N, sdata;
    output [6:0] charsig;

    reg  [12:0] state;
    wire [ 6:0] charsig;

    assign charsig = state[12:6];

  // state transition control
    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin
            state <= `S0;
        end else begin
            case(state)
              `S0    : state <= (sdata) ? `S2 : `S1;
              `S1    : state <= (sdata) ? `Sb : `Sa;
              `S2    : state <= (sdata) ? `S4 : `S3;
              `S3    : state <= (sdata) ? `Sd : `Sc;
              `S4    : state <= (sdata) ? `S5 : `Se;
              `S5    : state <= (sdata) ? `Sg : `Sf;
              `Sa,`Sb,
              `Sc,`Sd,
              `Se,`Sf,
              `Sg    : state <= (sdata) ? `S2 : `S1;
              default: state <= `SX;
            endcase
        end
    end

endmodule
