/* --------------------------------------------------------
 *  serial input variable length code decoder
 *	register decoder (Mealy type)
 *	(vldec_rl1.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

// state name definition
`define S0  3'b000
`define S1  3'b001
`define S2  3'b010
`define S3  3'b011
`define S4  3'b100
`define S5  3'b101
`define SX  3'b111

// character code definition
`define A   7'b1000000
`define B   7'b0100000
`define C   7'b0010000
`define D   7'b0001000
`define E   7'b0000100
`define F   7'b0000010
`define G   7'b0000001
`define NON 7'b0000000

module vldec(clock, reset_, sdata, charsig);
    input  clock, reset_, sdata;
    output [6:0] charsig;

    reg  [2:0] state;
    wire [6:0] charsig;

    assign charsig = chardet(state, sdata);

  // character detection logic
    function [6:0] chardet;
        input  [2:0] state;
        input        sdata;
        begin
            case (state) 
              `S1    : chardet = (sdata) ? `B   : `A;
              `S3    : chardet = (sdata) ? `D   : `C;
              `S4    : chardet = (sdata) ? `NON : `E;
              `S5    : chardet = (sdata) ? `G   : `F;
              default: chardet = `NON;
            endcase
        end
     endfunction

  // state transition control
    always @(posedge clock or negedge reset_) begin
        if(!reset_) begin
            state <= `S0;
        end else begin
            case (state)
              `S0    : state <= (sdata) ? `S2 : `S1;
              `S1    : state <= `S0;
              `S2    : state <= (sdata) ? `S4 : `S3;
              `S3    : state <= `S0;
              `S4    : state <= (sdata) ? `S5 : `S0;
              `S5    : state <= `S0;
              default: state <= `SX;
            endcase
        end
    end

endmodule
