/* --------------------------------------------------------
 *  serial input variable length code decoder
 *      register decoder version (Moore type)
 *	(vldec_rr2.v)		designed by Shinya KIMURA
 * -------------------------------------------------------- */

// state name definition
`define S0  4'b0000
`define S1  4'b0001
`define S2  4'b0010
`define S3  4'b0011
`define S4  4'b0100
`define S5  4'b0101
`define Sa  4'b1000
`define Sb  4'b1001
`define Sc  4'b1010
`define Sd  4'b1011
`define Se  4'b1100
`define Sf  4'b1101
`define Sg  4'b1110
`define SX  4'b0111

// character signal definition
`define A 7'b1000000
`define B 7'b0100000
`define C 7'b0010000
`define D 7'b0001000
`define E 7'b0000100
`define F 7'b0000010
`define G 7'b0000001

module vldec(clock, reset_N, sdata, charsig);
    input  clock, reset_N, sdata;
    output [6:0] charsig;

    reg  [4:0] state;
    wire [6:0] charsig;

    assign charsig = chardec(state[3:0]);
   
    function [6:0] chardec;
        input [3:0] state;
        begin
            case (state)
              `Sa    : chardec = `A;
              `Sb    : chardec = `B;
              `Sc    : chardec = `C;
              `Sd    : chardec = `D;
              `Se    : chardec = `E;
              `Sf    : chardec = `F;
              `Sg    : chardec = `G;
              default: chardec = 7'b0000000;
            endcase
        end
    endfunction

  // state transition control
    always @(posedge clock or negedge reset_N) begin
        if(!reset_N) begin
            state <= `S0;
        end else begin
            case(state)
              `S0    : state <= (sdata) ? `S2 : `S1;
              `S1    : state <= (sdata) ? `Sb : `Sa;
              `S2    : state <= (sdata) ? `S4 : `S3;
              `S3    : state <= (sdata) ? `Sd : `Sc;
              `S4    : state <= (sdata) ? `S5 : `Se;
              `S5    : state <= (sdata) ? `Sg : `Sf;
              `Sa,`Sb,
              `Sc,`Sd,
              `Se,`Sf,
              `Sg    : state <= (sdata) ? `S2 : `S1;
              default: state <= `SX;
            endcase
        end
    end
endmodule
